<?php

namespace Drupal\prompt_chatgpt;


/**
 * Class PromptService
 * @package Drupal\prompt_chatgpt\Services
 */
class PromptService {

  public function settings_fields(){
    return [
      'secret_key',
      'model',
      'role',
    ];
  }

  public function required_override_settings(): bool {
    return FALSE;
  }

  public function settings_form($override_config = []){
    $config = \Drupal::service('prompt.service')->override_config('prompt_chatgpt.settings', $override_config);

    $form['model'] = [
      '#type' => 'select',
      '#title' => t('Model'),
      '#description' => t('The model which will generate the completion.'),
      '#required' => true,
      '#options' => [
        'gpt-3.5-turbo' => 'gpt-3.5-turbo',
      ],
      '#default_value' => !empty($config->get('model')) ? $config->get('model') : 'gpt-3.5-turbo',
    ];

    $form['role'] = [
      '#type' => 'textarea',
      '#title' => t('Prompt Text for Personality role'),
      '#description' => t('You can specify which personality the AI has for this task. For example: "You are a helpful assistant that translates English to French". This helps ChatGPT to do a better job on the task.'),
      '#required' => false,
      '#default_value' => !is_null($config->get('role')) ? $config->get('role') : '',
    ];

    return $form;
  }

  function api_prompt_request($prompt, $config, $debug=FALSE){
    if(empty($prompt)){
      return '';
    }

    $secret_key = $config->get('secret_key');
    $model = $config->get('model');
    $role_content = $config->get('role');
    $messages = [];
    if(!empty($role_content)){
      $messages[] = [
        'role' => 'system',
        'content' => $role_content,
      ];
    }
    $messages[] = [
      'role' => 'user',
      'content' => $prompt,
    ];
    $request_body = [
      'model' => $model,
      'messages' => $messages,
    ];

    $postfields = json_encode($request_body);
    $curl = curl_init();
    curl_setopt_array($curl, [
      CURLOPT_URL => "https://api.openai.com/v1/chat/completions",
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_ENCODING => "",
      CURLOPT_MAXREDIRS => 10,
      CURLOPT_TIMEOUT => 60,
      CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
      CURLOPT_CUSTOMREQUEST => "POST",
      CURLOPT_POSTFIELDS => $postfields,
      CURLOPT_HTTPHEADER => [
        "Content-Type: application/json",
        "Authorization: Bearer $secret_key",
      ],
    ]);

    $response = curl_exec($curl);
    $err = curl_error($curl);

    curl_close($curl);

    if ($err) {
      \Drupal::messenger()->addMessage(
        t(
          'Error: @error', [
            '@error' => (string)$err,
          ]
        )
      );
    }
    else {
      $response = json_decode($response);

      if($debug == TRUE && !empty($response->usage)){
        \Drupal::messenger()->addMessage(
          t(
            'prompt_tokens:[@prompt_tokens], completion_tokens:[@completion_tokens]  -> total_tokens:[@total_tokens]', [
              '@prompt_tokens' => !empty($response->usage->prompt_tokens) ? $response->usage->prompt_tokens : 0,
              "@completion_tokens" => !empty($response->usage->completion_tokens) ? $response->usage->completion_tokens : 0,
              "@total_tokens" => !empty($response->usage->total_tokens) ? $response->usage->total_tokens : 0,
            ]
          ), 'warning');
      }

      if(!empty($response->error)){
        \Drupal::messenger()->addMessage(
          t(
            'Error: @error', [
              '@error' => (string)$response->error->message,
            ]
          ), 'error'
        );
      }
      else{
        return (string) $response->choices[0]->message->content;
      }
    }

    return NULL;

  }

}
